using System;
using System.Runtime.InteropServices;
using System.Threading;

namespace SSTPLib {
    /// <summary>
    /// DSSTPMNX
    /// </summary>
    public class DSSTPSender : IDisposable {
        #region Win32֐
        [StructLayout(LayoutKind.Sequential)]
        public struct COPYDATASTRUCT {
            public IntPtr dwData;
            public int cbData;
            public IntPtr lpData;
        }
        [DllImport("user32", CharSet = CharSet.Auto)]
        public extern static bool SendMessageTimeout(
            IntPtr hWnd,
            uint Msg,
            IntPtr wParam,
            ref COPYDATASTRUCT lParam,
            uint fuFlags,
            uint uTimeout,
            ref int lpdwResult
        );
        private const int SMTO_NORMAL = 0x0000;
        private const int SMTO_BLOCK = 0x0001;
        private const int SMTO_ABORTIFHUNG = 0x0002;
        private const int SMTO_NOTIMEOUTIFNOTHUNG = 0x0008;
        private const int WM_COPYDATA = 0x4A;
        private const int WM_DESTROY = 0x2;
        #endregion

        private SakuraFMO m_fmo = null;
        private FrmMsgReceiver m_frmMsgReceiver = null;
        private AutoResetEvent m_arevent = null;
        private Thread m_recvThread = null;
        private string m_result = null;
        private int m_sendmsgto = 3000;
        private int m_recvmsgto = 3000;

        #region RXgN^AfXgN^AvpeB
        /// <summary>
        /// RXgN^F"Sakura" FMO
        /// </summary>
        public DSSTPSender()
            : this("Sakura") {
        }

        /// <summary>
        /// RXgN^
        /// </summary>
        /// <param key="fmoName">FMO</param>
        public DSSTPSender(string fmoName) {
            m_fmo = new SakuraFMO(fmoName);
            m_fmo.Update(false);
            m_arevent = new AutoResetEvent(false);

            //create recv window thread
            //m_frmMsgReceiver=new FrmMsgReceiver();
            m_recvThread = new Thread(new ThreadStart(this.ThreadProc));
            m_recvThread.Priority = ThreadPriority.Lowest;
            m_recvThread.Start();
            m_arevent.WaitOne();
        }
        /// <summary>
        /// fXgN^
        /// </summary>
        ~DSSTPSender() {
            this.Dispose();
        }
        /// <summary>
        /// SSTP̖߂l擾܂
        /// </summary>
        public string Result {
            get { return m_result; }
        }
        /// <summary>
        /// SSTPM܂ł̃^CAEgԁi~bjݒE擾܂
        /// </summary>
        public int SendMessageTimeOut {
            get { return m_sendmsgto; }
            set { m_sendmsgto = value; }
        }
        /// <summary>
        /// SSTPMAʎM܂ł̃^CAEgԁi~bjݒE擾܂
        /// </summary>
        public int RecvMessageTimeOut {
            get { return m_recvmsgto; }
            set { m_recvmsgto = value; }
        }
        /// <summary>
        /// SSTPʎMp̕sEBhẼnh擾܂
        /// </summary>
        public IntPtr HWnd {
            get { return m_frmMsgReceiver.Handle; }
        }
        /// <summary>
        /// FMOǂݍݗpIFMOReader擾܂
        /// </summary>
        public IFMOReader FMO {
            get { return m_fmo; }
        }
        #endregion

        #region o֐
        /// <summary>
        /// FMO̓eēǂݍ݂܂
        /// </summary>
        /// <returns>ǂݍݐ^s</returns>
        public bool Update() {
            return m_fmo.Update(false);
        }
        /// <summary>
        /// SEND/1.4𑗐M܂
        /// </summary>
        /// <param key="sender">SSTP Sender p[^</param>
        /// <param key="ifghost">SSTP IfGhost p[^</param>
        /// <param key="script">SSTP Script p[^</param>
        /// <param key="keroname">FMOɎwIfGhost݂Ȃꍇ keroname w肵܂B
        /// FMOɑ݂ꍇFMO̓eŏ㏑邽߁AʏNULLō\܂B</param>
        /// <param key="isGetResult">SSTP̌ʂ҂ꍇTRUE ʂ҂ԂꍇFALSE</param>
        /// <returns>SSTPM̌ʂԂ܂</returns>
        public SSTPResult Send14(string sender, string ifghost, string script, string keroname, bool isGetResult) {
            uint hwnd;
            SakuraFMOData fd = m_fmo.GetGhostFMOData(ifghost);
            if (fd == null) {
                hwnd = m_fmo.GetGhostHWnd("");
            } else {
                hwnd = fd.hwnd;
                keroname = fd.keroname;
            }

            if (hwnd == 0) {
                return SSTPResult.ServerNotFound;
            }

            SSTPMessage msg = new SSTPMessage();
            msg.Method = SSTPMethod.SEND14;
            msg.Sender = sender;
            msg.Scripts = new SSTPIfGhostScript[] { new SSTPIfGhostScript(ifghost + "," + keroname, script) };
            msg.HWnd = (int)m_frmMsgReceiver.Handle;
            msg.XBottleIfGhost = ifghost;
            msg.CharSet = SSTPCharset.SHIFT_JIS;
            Byte[] data = msg.GetRequest();
            System.Diagnostics.Debug.WriteLine(msg.GetString());
            if (isGetResult) {
                return SendAndGetResult((IntPtr)hwnd, data);
            } else {
                bool res = DSSTPSender.SendWMCopyData((IntPtr)hwnd, m_frmMsgReceiver.Handle, data, this.SendMessageTimeOut);
                if (!res) {
                    return SSTPResult.SendMessageFailed;
                } else {
                    return SSTPResult.OK;
                }
            }
        }

        /// <summary>
        /// NOTIFY/1.1𑗐M܂
        /// </summary>
        /// <param key="sender">SSTP Sender p[^</param>
        /// <param key="ghost">MFMOsakura</param>
        /// <param key="notifyevent">SSTP Event p[^</param>
        /// <param key="references">SSTP Reference p[^</param>
        /// <param key="isGetResult">SSTP̌ʂ҂ꍇTRUE ʂ҂ԂꍇFALSE</param>
        /// <returns>SSTPM̌ʂԂ܂</returns>
        public SSTPResult Notify11(string sender, string ghost, string notifyevent, string[] references, bool isGetResult) {
            uint hwnd;
            hwnd = m_fmo.GetGhostHWnd(ghost);
            if (hwnd == 0) {
                hwnd = m_fmo.GetGhostHWnd(null);
            }
            if (hwnd == 0) {
                return SSTPResult.ServerNotFound;
            }
            SSTPMessage msg = new SSTPMessage();
            msg.Method = SSTPMethod.NOTIFY11;
            msg.Sender = sender;
            msg.Event = notifyevent;
            msg.References = references;
            msg.CharSet = SSTPCharset.SHIFT_JIS;
            msg.HWnd = (int)m_frmMsgReceiver.Handle;
            Byte[] data = msg.GetRequest();
            System.Diagnostics.Debug.WriteLine(msg.GetString());
            if (isGetResult) {
                return SendAndGetResult((IntPtr)hwnd, data);
            } else {
                bool res = DSSTPSender.SendWMCopyData((IntPtr)hwnd, m_frmMsgReceiver.Handle, data, this.SendMessageTimeOut);
                if (!res) {
                    return SSTPResult.SendMessageFailed;
                } else {
                    return SSTPResult.OK;
                }
            }
        }

        /// <summary>
        /// SSTP̓eSĎw肵đM܂
        /// </summary>
        /// <param key="ghost">MFMOsakura</param>
        /// <param key="sstpmsg">SSTP̓e</param>
        /// <param key="isGetResult">SSTP̌ʂ҂ꍇTRUE ʂ҂ԂꍇFALSE</param>
        /// <returns>SSTPM̌ʂԂ܂</returns>
        public SSTPResult SendSSTPString(string ghost, string sstpmsg, bool isGetResult) {
            uint hwnd;
            hwnd = m_fmo.GetGhostHWnd(ghost);
            if (hwnd == 0) {
                hwnd = m_fmo.GetGhostHWnd(null);
            }
            if (hwnd == 0) {
                return SSTPResult.ServerNotFound;
            }
            sstpmsg += "Charset: Shift_JIS\r\n";
            Byte[] data = System.Text.Encoding.GetEncoding("Shift-JIS").GetBytes(sstpmsg);
            if (isGetResult) {
                return SendAndGetResult((IntPtr)hwnd, data);
            } else {
                bool res = DSSTPSender.SendWMCopyData((IntPtr)hwnd, m_frmMsgReceiver.Handle, data, this.SendMessageTimeOut);
                if (!res) {
                    return SSTPResult.SendMessageFailed;
                } else {
                    return SSTPResult.OK;
                }
            }
        }

        /// <summary>
        /// SSTP̓eoCiŎw肵đMAʂ҂܂
        /// </summary>
        /// <param key="desthwnd">MHWND</param>
        /// <param key="data">Mf[^</param>
        /// <returns>SSTPM̌ʂԂ܂</returns>
        public SSTPResult SendAndGetResult(IntPtr desthwnd, Byte[] data) {
            bool result = DSSTPSender.SendWMCopyData(desthwnd, m_frmMsgReceiver.Handle, data, this.SendMessageTimeOut);
            if (!result) {
                return SSTPResult.SendMessageFailed;
            }
            bool waitresult = m_arevent.WaitOne(this.RecvMessageTimeOut, false);
            if (!waitresult) {
                return SSTPResult.ResultTimeOut;
            }
            m_result = System.Text.Encoding.GetEncoding("Shift-JIS").GetString(m_frmMsgReceiver.m_recvdata);
            System.Diagnostics.Debug.WriteLine(this.Result);
            string[] token = m_result.Split(new char[] { ' ' });
            if (token.Length < 3) {
                return SSTPResult.IllegalResultData;
            }
            int resultcode = 0;
            int val;
            bool result2 = int.TryParse(token[1], out val);
            if (result2) {
                resultcode = val;
            } else {
                return SSTPResult.IllegalResultData;
            }
            if (!Enum.IsDefined(typeof(SSTPResult), resultcode)) {
                return SSTPResult.UndefinedResultCode;
            }
            SSTPResult res = (SSTPResult)Enum.ToObject(typeof(SSTPResult), resultcode);
            return res;
        }

        /// <summary>
        /// WM_COPYDATA𑗐M܂BM܂ŏ̓ubN܂B
        /// </summary>
        /// <param key="desthwnd">MHWND</param>
        /// <param key="srchwnd">MHWND</param>
        /// <param key="data">Mf[^</param>
        /// <param key="timeout">M҂^CAEg</param>
        /// <returns>M^s</returns>
        public static bool SendWMCopyData(IntPtr desthwnd, IntPtr srchwnd, Byte[] data, int timeout) {
            IntPtr ptrData = Marshal.AllocCoTaskMem(data.Length);
            Marshal.Copy(data, 0, ptrData, data.Length);

            COPYDATASTRUCT cds = new COPYDATASTRUCT();
            cds.cbData = data.Length;
            cds.dwData = IntPtr.Zero;
            cds.lpData = ptrData;
            int resvalue = 0;

            bool result = SendMessageTimeout(desthwnd, WM_COPYDATA, srchwnd, ref cds,
                SMTO_ABORTIFHUNG | SMTO_BLOCK, (uint)timeout, ref resvalue);

            Marshal.FreeCoTaskMem(ptrData);

            return result;
        }

        /// <summary>
        /// WM_COPYDATA𑗐M܂BM܂ŏubN܂Bigpj
        /// </summary>
        /// <param key="desthwnd">MHWND</param>
        /// <param key="srchwnd">MHWND</param>
        /// <param key="data">Mf[^</param>
        /// <param key="timeout">M҂^CAEg</param>
        /// <returns>M^s</returns>
        public static bool SendWMCopyDataNoBlock(IntPtr desthwnd, IntPtr srchwnd, Byte[] data, int timeout) {
            IntPtr ptrData = Marshal.AllocCoTaskMem(data.Length);
            Marshal.Copy(data, 0, ptrData, data.Length);

            COPYDATASTRUCT cds = new COPYDATASTRUCT();
            cds.cbData = data.Length;
            cds.dwData = IntPtr.Zero;
            cds.lpData = ptrData;
            int resvalue = 0;

            bool result = SendMessageTimeout(desthwnd, WM_COPYDATA, srchwnd, ref cds,
                SMTO_ABORTIFHUNG, (uint)timeout, ref resvalue);

            Marshal.FreeCoTaskMem(ptrData);

            return result;
        }
        #endregion

        private void ThreadProc() {
            //System.Diagnostics.Debug.WriteLine("threadproc start");
            m_frmMsgReceiver = new FrmMsgReceiver();
            m_frmMsgReceiver.m_arevent = this.m_arevent;
            m_arevent.Set();
            System.Windows.Forms.Application.Run();
            //while(true){
            //	System.Windows.Forms.Application.DoEvents();
            //}
            //m_frmMsgReceiver.ShowDialog();
            //System.Diagnostics.Debug.WriteLine("threadproc end");
        }

        #region IDisposable o

        public void Dispose() {
            // TODO:  DSSTPSender.Dispose ǉ܂B
            System.Diagnostics.Debug.WriteLine("Dispose");
            if (m_frmMsgReceiver.Handle != IntPtr.Zero) {
                m_frmMsgReceiver.DestroyHandle();
            }
            if (m_recvThread != null) {
                m_recvThread.Join(1000);
                //				m_recvThread.Abort();
                m_recvThread = null;
            }
        }

        #endregion
    }
}
